# -*- coding: utf-8 -*-
import pytest, sys

sys.path.append("./pyettj/")
sys.path.append("./pyettj/pyettj/")

from ettj import get_ettj, plot_ettj, listar_dias_uteis
from modelo_ettj import get_ettj_anbima, svensson

data = "2021/05/18"

class TestClass():

    def test_ettj(self):
        self.ettj_dataframe = get_ettj(data)
        self.curvas = self.ettj_dataframe.columns.tolist()[1:]
        self.curva = self.curvas[2] #selic
        return self.ettj_dataframe, self.curva

    def test_listar_dias_uteis(self):
        de = "2021/05/13"
        ate = data
        datas=listar_dias_uteis(de, ate)
        with pytest.raises(Exception) as error4:
            de = 12
            ate = "2021/05/13"
            datas=listar_dias_uteis(de, ate)
        assert str(error4.value) == "O parametro data deve ser em formato string, exemplo: '18/05/2021'"
        with pytest.raises(Exception) as error5:
            de = "13/05/2021"
            ate = 34
            datas=listar_dias_uteis(de, ate)
        assert str(error5.value) == "O parametro data deve ser em formato string, exemplo: '18/05/2021'"

    def test_plot_ettj(self):
        self.ettj_dataframe, self.curva = self.test_ettj()
        plot_ettj(self.ettj_dataframe.drop(self.ettj_dataframe.columns[0],axis=1), self.curva, data)

    def test_raises(self):
        with pytest.raises(Exception) as error1:
            get_ettj(18)
        assert str(error1.value) == "O parametro data deve ser em formato string, exemplo: '18/05/2021'"
        with pytest.raises(Exception) as error2:
            get_ettj("teste")
        assert str(error2.value) == "O parametro data deve ser em formato string, exemplo: '18/05/2021'" 

    def test_svensson(self):
        taxa = svensson(0.13, 0.03, 0.03, 0.03, 1.97, 0.93, 1/252)
        assert round(taxa,2) == 0.16

    def get_ettj_anbima(self):
        data = "15/09/2022"
        parametros_curva, ettj, taxa, erros = get_ettj_anbima(data)
        assert not parametros_curva.empty
        assert not ettj.empty
        assert not taxa.empty
        assert not erros.empty

    def test_ettj2(self):
        data = "15/09/2022"
        curva = "pre"
        ettj_pre = get_ettj(data, curva)
        assert not ettj_pre.empty

TestClass()


import xml.etree.ElementTree as ET
import requests
import pandas as pd
import io
import numpy as np
from typing import Dict, Union

import warnings
warnings.filterwarnings("ignore")

def get_ettj_anbima(data: str, proxies: Union[Dict[str, str], None] = None) -> Union[pd.DataFrame, pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Captura dados ETTJ (PRE e IPCA) da ANBIMA.
        Parâmetros:
            data  (string) => data formato "%d/%m/%Y"
            proxies (dict) => opcional. se necessário, informar dicionário com as proxies, exemplo: {"http":f'https://{LOGIN}:{SENHA}@{PROXY_EMPRESA}:{PORTA}'}
        Retorno:
            parametros_curva (dataframe): parâmetros para montagem da curva svensson.
            ettj (dataframe): vértices e taxas por tipo de curva.
            taxa (dataframe): ettj da taxa prefixada.
            erro (dataframe): erro de estimação por tipo de Título Público.
    """
    url = "https://www.anbima.com.br/informacoes/est-termo/CZ-down.asp"
    payload = {"Idioma": "PT", "Dt_Ref": data, "saida": "xml"}
    headers = {"Content-type": "application/x-www-form-urlencoded", 
               "Accept":"text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9"}
    if proxies:
        ettj_anbima = requests.post(url, proxies=proxies, verify=False,
                                              data=payload,
                                              headers=headers).text
    else:
        ettj_anbima = requests.post(url,
                                    data=payload,
                                   headers=headers).text
        
    ettj_anbima_str = io.StringIO(ettj_anbima)

    tree1 = ET.parse(ettj_anbima_str)
    root1 = tree1.getroot()

    parametros_curva = pd.DataFrame()
    ettj = pd.DataFrame()
    taxa = pd.DataFrame()
    erros = pd.DataFrame()
    for x in tree1.iter():
        dado = x.attrib
        if x.tag == 'PARAMETRO': #parametros da curva
            curva = pd.DataFrame.from_dict(dado, orient="index").T
            parametros_curva = pd.concat([parametros_curva, curva])
        if x.tag == 'VERTICES': #ETTJ
            curva = pd.DataFrame.from_dict(dado, orient="index").T
            ettj = pd.concat([ettj, curva])
        if x.tag == 'CIRCULAR': #taxa por vertice
            curva = pd.DataFrame.from_dict(dado, orient="index").T
            taxa = pd.concat([taxa, curva])
        if x.tag == 'ERRO': #taxa por vertice
            curva = pd.DataFrame.from_dict(dado, orient="index").T
            erros = pd.concat([erros, curva])
    ettj = ettj.rename(columns={"Inflacao":"Inflação Implícita"})
    taxa = taxa.rename(columns={"Taxa":"Taxa Prefixada"})
    return parametros_curva.set_index("Grupo"), ettj, taxa, erros


def svensson(beta1: float, beta2: float, beta3: float, beta4: float, lambda1: float, lambda2: float, t: float) -> float:
    """Captura dados ETTJ (PRE e IPCA) da ANBIMA segundo equação Svensson (1994). 
        Para equação de Nelson Siegel (1987), basta informar beta4 e lambda2 iguais a zero.
        Parâmetros:
            beta1, beta2, beta3, beta4, lambda1 e lambda2 (float) => parâmetros que definem nivel e curvatura;
            t (float) => maturidade/vértice em anos.
        Retorno:
            taxa (float): taxa de juros da curva.
    """    
    def exponencial(lambdas, t):
        return np.exp(1)**(-lambdas*t)
    
    return beta1 + beta2* ((1-exponencial(lambda1, t))/(lambda1*t)) + \
                    beta3* ((1-exponencial(lambda1, t))/(lambda1*t) - exponencial(lambda1, t)) + \
                       beta4* ((1-exponencial(lambda2, t))/(lambda2*t) - exponencial(lambda2, t))